from ucfg_configurator import *
from ucfg_utils import info, error
from ucfg_main import ConfigUtilities


class UConfigurator(UConfiguratorBase):
	__gw_params = {
			'gateway.hostname'	   			: 'https://0.0.0.0:8080##URL on which server will listen',
			'gateway.registration.enable'   : 'false##Whether to allow for autoregistration of backend sites',
			'gateway.connectionEntry.0'  	: 'DEMO-SITE = https://localhost:7777##List of fixed backend sites'
	}

	def getParams(self):
		p = self.addCredAndTrustParams('gateway', self.__gw_params)
		p = self.addStartupParams('gateway', p, 128);
		return p

	def run(self, options, config, section, params):
		info(options, "UNICORE Gateway configurator (" + section + ")")
		cfgUtil = ConfigUtilities(options, section)

		# connections.properties
		connectionEntriesDict = { '${uxName}' : None }
		i = 0
		while 'gateway.connectionEntry.%d' % i in params.keys():
			connEntry = params['gateway.connectionEntry.%d' % i].split('=', 1)
			if len(connEntry) == 2:
				connKey = connEntry[0].strip()
				connVal = connEntry[1].strip()
				connectionEntriesDict[connKey] = connVal
			else:
				error("Wrong 'gateway.connectionEntry.%d' entry. Skipping it..." % i)
			i = i + 1
		cfgUtil.setShellVariables('connections.properties', connectionEntriesDict)

		# gateway.properties
		cfgUtil.setJavaProperties('gateway.properties', {
			'gateway.hostname' 				: params['gateway.hostname'],
			'gateway.registration.enable' 	: params['gateway.registration.enable']
		})

		# security.properties
		cfgUtil.setJavaProperties('security.properties', self.prepareCredAndTrustDict('gateway', params))
		
		# startup.properties
		cfgUtil.setShellVariables('startup.properties', self.prepareStartupDict('gateway', params))

		info(options, "Finished UNICORE Gateway (" + section + ")")

