import ucfg_utils
from ucfg_utils import info, error
from ucfg_main import ConfigUtilities

from unicorex import UConfigurator as UXConfigurator
import getpass
import os


class UConfigurator(UXConfigurator):
	__plg_params = {
		# plg parameters:
		#
		'gwHostName' 		: '__REQUIRED',
		'uxSiteName' 		: '__REQUIRED',
		'plgSiteAbbr' 		: '__REQUIRED',

		'plgIDBPath' 		: '__REQUIRED',
		'plgGroupsPath'		: '__REQUIRED',
		'plgSiteProtocols' 	: 'BFT RBYTEIO SBYTEIO',

		'gocDbSiteName' 	: '__REQUIRED',
		'plgBssMachine'		: '__REQUIRED',
		'plgAcctUsername'	: '__REQUIRED',
		'plgAcctPassword'	: '__REQUIRED',

        'plgEnableReservations' : 'false',
		'reservationAdminUser'  : None,

		'mysqlHost'			: '__REQUIRED',
		'mysqlPort'			: '__REQUIRED',
		'mysqlDb'			: '__REQUIRED',
		'mysqlUser'			: '__REQUIRED',
		'mysqlPassword'		: '__REQUIRED',

		# override unicorex parameters:
		#
			# startup.properties
		'useJmxPort' 			: '',
		'useMemory'				: '512m',
			# uas.config
		'uxDisableHomes'        : 'false',
		'uxXNJSConfigFile' 		: '/etc/unicore/unicorex/xnjs_legacy.xml',
		'uxUseExternalRegistry' : 'true',
		'uxUrlExternalRegistry' : 'https://unicore.grid.icm.edu.pl:8080/PLGRID/services/Registry?res=default_registry',
		'uxEnableGlobalStorage' : 'true',
		'uxGlobalStoragePath'	: '__REQUIRED',
			# xnjs_legacy.xml
		'uxTSIDirectory'		: '/usr/share/unicore/tsi_selected',

		# override _use parameters:
		#
			# wsrflite.xml
		'useKeystorePath' 		: '__REQUIRED',
		'useKeystorePassword'	: '__REQUIRED',
		'useKeystoreAlias' 		: '__REQUIRED',
		'useTruststorePath' 	: '__REQUIRED',
		'useTruststorePassword'	: '__REQUIRED'
	}


	def __init__(self):
		super(UConfigurator, self).__init__()
		self.__my_params = super(UConfigurator, self).getParams().copy()
		self.__my_params.update(self.__plg_params)

	def getParams(self):
		return self.__my_params

	def run(self, options, config, section, params):
		info(options, "PL-Grid UNICORE/X Configurator (" + section + ")")

		#remove what we don't want to have (in gateway and unicorex directory)
		self.__removeUnnecessaryConfigurationFiles(options)
		#fix startup scripts (for gateway and unicorex)
		self.__fixStartupScripts(options)

		#update parameters for unicorex configurator			
		if 'plgHostIP' in params.keys():
			params['uftpHost'] = params['plgHostIP']
		else:
			params['uftpHost'] = params['gwHostName']

		#apply standard unicorex configuration
		super(UConfigurator, self).run(options, config, 'unicorex', params)
		#create helper object handling config files processing
		cfgUtil = ConfigUtilities(options, 'unicorex')

		#process plg-specific configuration
		self.__processUnicoreXConfiguration(options, cfgUtil, params)
		if params['plgEnableReservations'] == 'true':
			self.__enableReservationSetup(options, cfgUtil)
			cfgUtil.setXNJSProperties('xnjs_legacy.xml', {
				'CLASSICTSI.reservationAdminUser' :  params['reservationAdminUser']
			})

		info(options, "Finished PL-Grid UNICORE/X (" + section + ")")



	def __processUnicoreXConfiguration(self, options, cfgUtil, params):

		cfgUtil.setJavaProperty('logging.properties',
				'log4j.appender.A1.layout.ConversionPattern', '%d [%t] %-5p %c %x - %m%n')

		cfgUtil.removeXPathElements('simpleuudb',
			"/fileAttributeSource/entry[@key='EMAILADDRESS=unicore-support@lists.sf.net, C=DE, O=unicore.eu, OU=Testing, CN=UNICORE demo user']")

		#persistence.properties
		cfgUtil.setJavaProperties('persistence.properties', {
			'persistence.class' 	: 'de.fzj.unicore.persist.impl.MySQLPersist',
			'persistence.host' 		: params['mysqlHost'],
			'persistence.port' 		: params['mysqlPort'],
			'persistence.database' 	: params['mysqlDb'],
			'persistence.user' 		: params['mysqlUser'],
			'persistence.password' 	: params['mysqlPassword'],
			'persistence.cluster.enable' : 'false'
		})

		#uas.config stuff		
		cfgUtil.setJavaProperties('uas.config', {
			'uas.externalregistry.url.2' : 'https://lalande.wcss.pl:8080/PLGRID/services/Registry?res=default_registry',
			'uas.onstartup.wait' : 'true',
			'uas.security.consignor.checksignature' : 'false',
			'uas.targetsystem.storage.force_unique_ids' : 'true',
			'uas.sms.protocols' : params['plgSiteProtocols'],
			'defaultsms.name' : 'SHARE-' + params['plgSiteAbbr'],
			'defaultsms.protocols' : params['plgSiteProtocols']
		})
		cfgUtil.setJavaProperties('uas.config', {
			'uas.security.attributes.order' : 'FILE VO-PULL-CLUSTER',
			'uas.security.attributes.combiningPolicy' : 'FIRST_APPLICABLE',
			'uas.security.attributes.VO-PULL-CLUSTER.class' : 'de.fzj.unicore.wsrflite.security.util.AttributeSourcesChain',
			'uas.security.attributes.VO-PULL-CLUSTER.order' : 'VO-PULL-ICM VO-PULL-WCSS',
			'uas.security.attributes.VO-PULL-CLUSTER.combiningPolicy' : 'FIRST_ACCESSIBLE',
			'uas.security.attributes.VO-PULL-ICM.class' : 'eu.unicore.uas.security.vo.SAMLPullAuthoriser',
			'uas.security.attributes.VO-PULL-ICM.configurationFile' : '/etc/unicore/unicorex/vo-icm.config',
			'uas.security.attributes.VO-PULL-WCSS.class' : 'eu.unicore.uas.security.vo.SAMLPullAuthoriser',
			'uas.security.attributes.VO-PULL-WCSS.configurationFile' : '/etc/unicore/unicorex/vo-wcss.config',
			'uas.security.attributes.VO-PUSH.class' : None,
			'uas.security.attributes.VO-PUSH.configurationFile' : None,

			# [2012-10-24] VO configuration now handled at site (must be a child of site's vo.group path):
			'uas.security.defaultVOs' : '/vo.plgrid.pl/main/' + params['plgSiteAbbr']
		})

		cfgUtil.setJavaProperties('uas.config', {
			'uftp.server.host' 		 : params['uftpHost'],
			'uftp.server.port' 		 : '64434',
			'uftp.command.host' 	 : 'localhost',
			'uftp.command.port' 	 : '64435',
			'uftp.client.executable' : '/usr/bin/uftp.sh',
			'uftp.streams' 			 : '2',
			'uftp.buffersize' 		 : '4096',
			'uftp.encryption' 		 : 'true'
		})
		if params['plgGroupsPath'] != '':
			cfgUtil.setJavaProperties('uas.config', {
				'uas.targetsystem.storage.1' 				: 'GROUPS',
				'uas.targetsystem.storage.1.type' 			: 'FIXEDPATH',
				'uas.targetsystem.storage.1.path' 			: params['plgGroupsPath'],
				'uas.targetsystem.storage.1.protocols' 		: params['plgSiteProtocols'],
				'uas.targetsystem.storage.1.filterFiles' 	: 'true',
				'uas.targetsystem.storage.1.defaultUmask' 	: '7'
			})
		else:
			cfgUtil.setJavaProperties('uas.config', {
				'uas.targetsystem.storage.1' 				: None,
				'uas.targetsystem.storage.1.type' 			: None,
				'uas.targetsystem.storage.1.path' 			: None,
				'uas.targetsystem.storage.1.protocols' 		: None,
				'uas.targetsystem.storage.1.filterFiles' 	: None,
				'uas.targetsystem.storage.1.defaultUmask' 	: None
			})


		#wsrflite.xml
		cfgUtil.removeXPathElements("wsrflite.xml", "/services/property[@name='persistence.directory']")
		plgBaseUrl = 'https://' + params['gwHostName'] + ':8080/' + params['uxSiteName'] + '/services'
		cfgUtil.setWSRFLiteProperties('wsrflite.xml', {
			'unicore.wsrflite.baseurl' : plgBaseUrl,

			'wsrflite.lifetime.default.JobManagement' 		  :  '5184000', # 60d
			'wsrflite.lifetime.default.ReservationManagement' : '15552000', # 180d
			'wsrflite.lifetime.default.StorageManagement'     :  '8640000', # 100d
			'wsrflite.lifetime.default.TargetSystemService'   : '17280000', # 200d

			'unicore.maxInstancesPerUser.JobManagement'         : '2000',
			'unicore.maxInstancesPerUser.ReservationManagement' :   '10',
			'unicore.maxInstancesPerUser.StorageManagement'     : '5000',
			'unicore.maxInstancesPerUser.TargetSystemService'   :    '3',

			# [2012-10-31] Set to enable DOS attacks: 
			'unicore.maxInstancesPerUser.BESActivity'         : '200',
			'unicore.maxInstancesPerUser.FileTransfer'        : '200',
			'unicore.maxInstancesPerUser.FileTransferBFT'     : '200',
			'unicore.maxInstancesPerUser.FileTransferRBYTEIO' : '200',
			'unicore.maxInstancesPerUser.FileTransferSBYTEIO' : '200',
			'unicore.maxInstancesPerUser.FileTransferUDT'     : '200',
			'unicore.maxInstancesPerUser.FileTransferUFTP'    : '200',

			'persistence.config' : '/etc/unicore/unicorex/persistence.properties'
		})
		uftpServiceSwitch = 'false'
		if 'UFTP' in params['plgSiteProtocols']:
			uftpServiceSwitch = 'true'
		cfgUtil.toggleWSRFLiteServices('wsrflite.xml', {
			'DelegationService': 'false',
			'CreateActivityService': 'false',
			'ActivityManagementService': 'false',
			'ActivityInfoService': 'false',
			'ResourceInfoService': 'false',
			'FileTransferUFTP' : uftpServiceSwitch,
			'ReservationManagement' : 'false',
			# [2012-10-24] Enabling AdminService for metrics (must have admin role): 
			'AdminService': 'true'
		})

		#xnjs_legacy.xml
		cfgUtil.setXNJSProperties('xnjs_legacy.xml', {
			'XNJS.idbfile' 					: params['plgIDBPath'],
			'XNJS.incarnationTweakerConfig' : '/etc/unicore/unicorex/incarnationTweaker.xml',

			# [2012-10-10] Set number of resubmissions to BSS in case of errors:
			'XNJS.bssResubmitCount'         : '2',
			'XNJS.bssResubmitDelay'         : '15',

			'CLASSICTSI.BUFFERSIZE' 		: '1048576',
			# File System ID - used by local copy optimization, so if TSI listens on localhost,
			# it should be different on different machines. 
			'CLASSICTSI.FSID'               : 'PERL TSI at ' + params['gwHostName'],

			'persistence.config' 			: '/etc/unicore/unicorex/persistence.properties',

			# UNICORE RUS accounting settings:
			'RUS.site.name' 						: params['gocDbSiteName'],
			'RUS.ce.node.name' 						: params['gwHostName'],
			'RUS.bssMachine' 						: params['plgBssMachine'],
			'RUS.PROCESSOR.jms.url' 				: 'ssl://unicore.grid.icm.edu.pl:61617?soTimeout=3000',
			'RUS.PROCESSOR.jms.queue' 				: 'plgrid.q',
			'RUS.PROCESSOR.jms.username' 			: params['plgAcctUsername'],
			'RUS.PROCESSOR.jms.password' 			: params['plgAcctPassword'],
			'RUS.PROCESSOR.jms.keystore' 			: params['useKeystorePath'],
			'RUS.PROCESSOR.jms.keystore.password' 	: params['useKeystorePassword'],
			'RUS.PROCESSOR.jms.truststore' 			: '/etc/unicore/grid-security/plg-truststore.jks',
			'RUS.PROCESSOR.jms.truststore.password' : 'plgrid'
		})
		cfgUtil.appendXNJSProcessor('xnjs_legacy.xml',
								'pl.edu.icm.unicore.accounting.processor.AccountingJobProcessor')

		# vo configs
		cfgUtil.setJavaProperties('vo-icm.config', {
			'vo.group' : '/vo.plgrid.pl/main/' + params['plgSiteAbbr'],
			'vo.localServerURI' : 'https://' + params['gwHostName'] + ':7777/' + params['uxSiteName']
		})
		cfgUtil.setJavaProperties('vo-wcss.config', {
			'vo.group' : '/vo.plgrid.pl/main/' + params['plgSiteAbbr'],
			'vo.localServerURI' : 'https://' + params['gwHostName'] + ':7777/' + params['uxSiteName']
		})



	def __fixStartupScripts(self, options):
        # fixing starting scripts
		for startScript in [ '/usr/sbin/unicore-gateway-start.sh',
							'/usr/sbin/unicore-unicorex-start.sh' ]:
			if os.path.isfile(startScript):
				try:
					ucfg_utils.setShellVariables(options, startScript, {
						'CP' : '.$(cd /; find "${LIB}" -name *.jar -exec printf ":{}" \;)'
					})
				except IOError, (errno, strerror):
					error("Can't modify start-up script '%s': I/O error(%s): %s" % (startScript, errno, strerror))



	def __removeUnnecessaryConfigurationFiles(self, options):
		from ucfg_raw import removeUnnecessaryFiles
		removeUnnecessaryFiles(options, 'gateway', [
												'gateway_keystore.jks',
												'wrapper.conf',
												'wrapper.conf.template'
								])
		removeUnnecessaryFiles(options, 'unicorex', [
												'keystore.jks',
												'ucc.extensions',
												'ucc.logging.properties',
												'ucc.preferences',
												'vo.config',
												'votruststore.jks',
												'wrapper.conf',
												'wrapper.conf.template',
												'xnjs.xml'
								])
		removeUnnecessaryFiles(options, 'uftpd', [
												'client.logging.properties',
												'uftpd.jks'
								])
		removeUnnecessaryFiles(options, 'root', [
												#'/usr/share/unicore/unicorex/lib/xnjs-module-core-1.5.2.jar', 
												#'/usr/share/unicore/unicorex/lib/uas-core-1.5.0.jar',

												#'/usr/share/unicore/unicorex/lib/rus-commons-1.5.0.jar',
												#'/usr/share/unicore/unicorex/lib/rus-job-processor-1.5.0.jar',
												#'/usr/share/unicore/unicorex/lib/rus-types-1.5.1.jar',

												'/usr/share/unicore/unicorex/lib/rus-commons-1.4.0.jar',
												'/usr/share/unicore/unicorex/lib/rus-job-processor-1.4.0.jar',
												'/usr/share/unicore/unicorex/lib/rus-service-1.4.0.jar'
								])



	def __enableReservationSetup(self, options, cfgUtil):
		from ucfg_utils import info
		info(options, 'Starting configuration of reservation service')
		options.indent = options.indent + "  "

		xnjsReservationProcessor = '<eng:Reservation xmlns:eng="http://www.fz-juelich.de/unicore/xnjs/engine">de.fzj.unicore.xnjs.legacy.Reservation</eng:Reservation>'
		nsPrefixMap = { 'eng' : 'http://www.fz-juelich.de/unicore/xnjs/engine' }

		cfgUtil.removeXPathElements('xnjs_legacy.xml',
								'/eng:EngineConfiguration/eng:Core/eng:Reservation',
								nsPrefixMap)
		cfgUtil.addXMLStringElementAfterXPath('xnjs_legacy.xml',
											xnjsReservationProcessor,
											'/eng:EngineConfiguration/eng:Core/eng:ProcessorFactory',
											nsPrefixMap)
		info(options, ' - added XNJS reservation configuration')

		wsrfReservationService = '<service name="ReservationManagement" wsrf="true" persistent="true" enabled="true">' \
			+ 	'<interface class="de.fzj.unicore.uas.ReservationManagement" />' \
			+ 	'<implementation class="de.fzj.unicore.uas.impl.reservation.ReservationManagementHomeImpl"/>' \
			+ '</service>'

		cfgUtil.removeXPathElements('wsrflite.xml',
								"/services/service[@name='ReservationManagement']"
							)
		cfgUtil.addXMLStringElementAfterXPath('wsrflite.xml',
											wsrfReservationService
										)
		info(options, ' - added correct WSRF reservation service')

		options.indent = options.indent[:-2]
		info(options, 'Configuration of reservation service finished')

