#!/usr/bin/python
'''
	This file runs number of tests which are checking XPath operations of lxml library.
	If everything is OK, configurator should work correctly.
'''
import sys
import tempfile, os

from lxml import etree

'''
	Checking xml operations on wsrflite.xml template
'''
print "Checking operations on WSRFLite configuration example..."

wsrfLiteXml = '<?xml version="1.0" encoding="utf-8"?> \
<services> <!-- UNICORE/X truststore (can be the same as the keystore) --> \
  <property name="unicore.wsrflite.ssl.truststore" value="../cert/truststore.jks"/> \
  <property name="unicore.wsrflite.ssl.truststoretype" value="JKS"/> \
  <!-- the UNICORE core services --> \
  <property name="unicore.maxInstancesPerUser.TargetSystemService" value="3"/> \
  <property name="unicore.maxInstancesPerUser.JobManagement" value="20"/> \
  <!-- \
   <property name="unicore.maxInstancesPerUser.JobManagement" value="200"/> \
   <property name="unicore.maxInstancesPerUser.StorageManagement" value="250"/> \
   --> \
  <!-- [...] --> \
  <!-- the wsrflite admin service --> \
  <service name="AdminService" wsrf="true"> \
    <interface class="de.fzj.unicore.wsrflite.admin.service.AdminService"/> \
    <implementation class="de.fzj.unicore.wsrflite.admin.service.AdminServiceHomeImpl"/></service> \
</services>'
wsrfParser = etree.XMLParser(remove_blank_text = True, ns_clean = True, remove_comments = True)
wsrfDoc = etree.fromstring(wsrfLiteXml, wsrfParser)

print ' - removing property element based on XPath'
tsTypeElements = wsrfDoc.xpath("/services/property[@name='unicore.wsrflite.ssl.truststoretype']")
assert len(tsTypeElements) == 1
tsTypeParent = tsTypeElements[0].getparent()
assert tsTypeParent != None
tsTypeParent.remove(tsTypeElements[0])

wsrfXmlResultExpected = '<services>\n\
  <property name="unicore.wsrflite.ssl.truststore" value="../cert/truststore.jks"/>\n\
  <property name="unicore.maxInstancesPerUser.TargetSystemService" value="3"/>\n\
  <property name="unicore.maxInstancesPerUser.JobManagement" value="20"/>\n\
  <service name="AdminService" wsrf="true">\n\
    <interface class="de.fzj.unicore.wsrflite.admin.service.AdminService"/>\n\
    <implementation class="de.fzj.unicore.wsrflite.admin.service.AdminServiceHomeImpl"/>\n\
  </service>\n\
</services>\n'
res = etree.tostring(wsrfDoc,
	encoding = 'utf-8', method = 'xml', pretty_print = True, xml_declaration = False)
assert res == wsrfXmlResultExpected
print ' - result: OK'

print ' - setting property element based on XPath'
tssMaxProperty = wsrfDoc.xpath("/services/property[@name='unicore.maxInstancesPerUser.TargetSystemService']")
assert len(tssMaxProperty) == 1
tssMaxProperty[0].set('value', '7')
wsrfXmlResultExpected = '<services>\n\
  <property name="unicore.wsrflite.ssl.truststore" value="../cert/truststore.jks"/>\n\
  <property name="unicore.maxInstancesPerUser.TargetSystemService" value="7"/>\n\
  <property name="unicore.maxInstancesPerUser.JobManagement" value="20"/>\n\
  <service name="AdminService" wsrf="true">\n\
    <interface class="de.fzj.unicore.wsrflite.admin.service.AdminService"/>\n\
    <implementation class="de.fzj.unicore.wsrflite.admin.service.AdminServiceHomeImpl"/>\n\
  </service>\n\
</services>\n'
res = etree.tostring(wsrfDoc,
	encoding = 'utf-8', method = 'xml', pretty_print = True, xml_declaration = False)
assert res == wsrfXmlResultExpected
print ' - result: OK'

print " - setting service attribute 'enabled'"
adminServiceProperty = wsrfDoc.xpath("/services/service[@name='AdminService']")
assert len(adminServiceProperty) == 1
adminServiceProperty[0].set('enabled', 'true')
wsrfXmlResultExpected = '<services>\n\
  <property name="unicore.wsrflite.ssl.truststore" value="../cert/truststore.jks"/>\n\
  <property name="unicore.maxInstancesPerUser.TargetSystemService" value="7"/>\n\
  <property name="unicore.maxInstancesPerUser.JobManagement" value="20"/>\n\
  <service name="AdminService" wsrf="true" enabled="true">\n\
    <interface class="de.fzj.unicore.wsrflite.admin.service.AdminService"/>\n\
    <implementation class="de.fzj.unicore.wsrflite.admin.service.AdminServiceHomeImpl"/>\n\
  </service>\n\
</services>\n'
res = etree.tostring(wsrfDoc,
	encoding = 'utf-8', method = 'xml', pretty_print = True, xml_declaration = False)
assert res == wsrfXmlResultExpected
print ' - result: OK'

print ' - adding service entry before element determined by XPath'
goofyServiceXml = '<service name="GoofyService" wsrf="true" enabled="false"></service>'
goofyElement = etree.fromstring(goofyServiceXml)
adminServiceElement = wsrfDoc.xpath("/services/service[@name='AdminService']")
assert len(adminServiceElement) == 1
adminServiceElement[0].addprevious(goofyElement)
wsrfXmlResultExpected = '<services>\n\
  <property name="unicore.wsrflite.ssl.truststore" value="../cert/truststore.jks"/>\n\
  <property name="unicore.maxInstancesPerUser.TargetSystemService" value="7"/>\n\
  <property name="unicore.maxInstancesPerUser.JobManagement" value="20"/>\n\
  <service name="GoofyService" wsrf="true" enabled="false"/>\n\
  <service name="AdminService" wsrf="true" enabled="true">\n\
    <interface class="de.fzj.unicore.wsrflite.admin.service.AdminService"/>\n\
    <implementation class="de.fzj.unicore.wsrflite.admin.service.AdminServiceHomeImpl"/>\n\
  </service>\n\
</services>\n'
res = etree.tostring(wsrfDoc,
	encoding = 'utf-8', method = 'xml', pretty_print = True, xml_declaration = False)
assert res == wsrfXmlResultExpected
print ' - result: OK'



'''
	Checking xml operations on xnjs_legacy.xml template
'''
print "Checking operations on XNJS configuration example..."
xnjsNSPrefixMap = { 'eng' : 'http://www.fz-juelich.de/unicore/xnjs/engine' }

xnjsXmlTemplate = '<?xml version="1.0" encoding="utf-8"?> \
<eng:EngineConfiguration xmlns:eng="http://www.fz-juelich.de/unicore/xnjs/engine"> \
  <eng:Core> \
    <!-- [...] --> \
    <eng:ProcessorFactory>de.fzj.unicore.xnjs.ems.processors.DefaultProcessorFactory</eng:ProcessorFactory> \
    <!-- reservation interface, uncomment to use the TSI reservation functions (Reservation.pm) --> \
    <!-- <eng:Reservation>de.fzj.unicore.xnjs.legacy.Reservation</eng:Reservation> --> \
    <!-- i/o --> \
    <eng:FileTransferEngine>de.fzj.unicore.xnjs.io.simple.SimpleFileTransferEngine</eng:FileTransferEngine> \
  </eng:Core> \
  <eng:ProcessingChain actionType="JSDL" jobDescriptionType="{http://schemas.ggf.org/jsdl/2005/11/jsdl}JobDefinition"> \
    <eng:Processor>de.fzj.unicore.xnjs.jsdl.JSDLProcessor</eng:Processor> \
    <eng:Processor>de.fzj.unicore.xnjs.ems.processors.UsageLogger</eng:Processor> \
  </eng:ProcessingChain> \
  <!-- [...] --> \
  <eng:Properties> \
    <!-- [...] --> \
    <eng:Property name="RUS.bssMachine" value="localhost"/> \
  </eng:Properties> \
</eng:EngineConfiguration>'

print ' - checking temporary file creation and its write access'
fd, filename = tempfile.mkstemp()
assert os.path.isfile(filename) == True
fh = os.fdopen(fd, 'w')
fh.write(xnjsXmlTemplate)
fh.close()
assert os.stat(filename).st_size > 0
print ' - result: OK'

print ' - reading xml from a file and unlinking it'
xnjsParser = etree.XMLParser(remove_blank_text = True, ns_clean = True, remove_comments = True)
xnjsDoc = etree.parse(filename, xnjsParser)
os.unlink(filename)
assert os.path.exists(filename) == False
xnjsXmlResultExpected = '<eng:EngineConfiguration xmlns:eng="http://www.fz-juelich.de/unicore/xnjs/engine">\n\
  <eng:Core>\n\
    <eng:ProcessorFactory>de.fzj.unicore.xnjs.ems.processors.DefaultProcessorFactory</eng:ProcessorFactory>\n\
    <eng:FileTransferEngine>de.fzj.unicore.xnjs.io.simple.SimpleFileTransferEngine</eng:FileTransferEngine>\n\
  </eng:Core>\n\
  <eng:ProcessingChain actionType="JSDL" jobDescriptionType="{http://schemas.ggf.org/jsdl/2005/11/jsdl}JobDefinition">\n\
    <eng:Processor>de.fzj.unicore.xnjs.jsdl.JSDLProcessor</eng:Processor>\n\
    <eng:Processor>de.fzj.unicore.xnjs.ems.processors.UsageLogger</eng:Processor>\n\
  </eng:ProcessingChain>\n\
  <eng:Properties>\n\
    <eng:Property name="RUS.bssMachine" value="localhost"/>\n\
  </eng:Properties>\n\
</eng:EngineConfiguration>\n'
xnjsResult = etree.tostring(xnjsDoc,
	encoding = 'utf-8', method = 'xml', pretty_print = True, xml_declaration = False)
assert xnjsResult == xnjsXmlResultExpected
print ' - result: OK'

print ' - changing property with namespace'
elementXPath = "/eng:EngineConfiguration/eng:Properties/eng:Property[@name='RUS.bssMachine']"
elements = xnjsDoc.xpath(elementXPath, namespaces = xnjsNSPrefixMap)
assert len(elements) == 1
elements[0].set('value', 'goofy.grid')
xnjsXmlResultExpected = '<eng:EngineConfiguration xmlns:eng="http://www.fz-juelich.de/unicore/xnjs/engine">\n\
  <eng:Core>\n\
    <eng:ProcessorFactory>de.fzj.unicore.xnjs.ems.processors.DefaultProcessorFactory</eng:ProcessorFactory>\n\
    <eng:FileTransferEngine>de.fzj.unicore.xnjs.io.simple.SimpleFileTransferEngine</eng:FileTransferEngine>\n\
  </eng:Core>\n\
  <eng:ProcessingChain actionType="JSDL" jobDescriptionType="{http://schemas.ggf.org/jsdl/2005/11/jsdl}JobDefinition">\n\
    <eng:Processor>de.fzj.unicore.xnjs.jsdl.JSDLProcessor</eng:Processor>\n\
    <eng:Processor>de.fzj.unicore.xnjs.ems.processors.UsageLogger</eng:Processor>\n\
  </eng:ProcessingChain>\n\
  <eng:Properties>\n\
    <eng:Property name="RUS.bssMachine" value="goofy.grid"/>\n\
  </eng:Properties>\n\
</eng:EngineConfiguration>\n'
xnjsResult = etree.tostring(xnjsDoc,
	encoding = 'utf-8', method = 'xml', pretty_print = True, xml_declaration = False)
assert xnjsResult == xnjsXmlResultExpected
print ' - result: OK'

print ' - appending processor in JSDL processing chain'
acctProcessorXML = '<eng:Processor xmlns:eng="http://www.fz-juelich.de/unicore/xnjs/engine">pl.edu.icm.unicore.accounting.processor.AccountingJobProcessor</eng:Processor>'
acctProcessorEl = etree.fromstring(acctProcessorXML, xnjsParser)
usageProcessorXPath = "/eng:EngineConfiguration/eng:ProcessingChain/eng:Processor[text()='de.fzj.unicore.xnjs.ems.processors.UsageLogger']"
processorElements = xnjsDoc.xpath(usageProcessorXPath, namespaces = xnjsNSPrefixMap)
assert len(processorElements) == 1
processorElements[0].addnext(acctProcessorEl)
xnjsXmlResultExpected = '<eng:EngineConfiguration xmlns:eng="http://www.fz-juelich.de/unicore/xnjs/engine">\n\
  <eng:Core>\n\
    <eng:ProcessorFactory>de.fzj.unicore.xnjs.ems.processors.DefaultProcessorFactory</eng:ProcessorFactory>\n\
    <eng:FileTransferEngine>de.fzj.unicore.xnjs.io.simple.SimpleFileTransferEngine</eng:FileTransferEngine>\n\
  </eng:Core>\n\
  <eng:ProcessingChain actionType="JSDL" jobDescriptionType="{http://schemas.ggf.org/jsdl/2005/11/jsdl}JobDefinition">\n\
    <eng:Processor>de.fzj.unicore.xnjs.jsdl.JSDLProcessor</eng:Processor>\n\
    <eng:Processor>de.fzj.unicore.xnjs.ems.processors.UsageLogger</eng:Processor>\n\
    <eng:Processor>pl.edu.icm.unicore.accounting.processor.AccountingJobProcessor</eng:Processor>\n\
  </eng:ProcessingChain>\n\
  <eng:Properties>\n\
    <eng:Property name="RUS.bssMachine" value="goofy.grid"/>\n\
  </eng:Properties>\n\
</eng:EngineConfiguration>\n'
xnjsResult = etree.tostring(xnjsDoc,
	encoding = 'utf-8', method = 'xml', pretty_print = True, xml_declaration = False)
assert xnjsResult == xnjsXmlResultExpected
print ' - result: OK'

print ' - adding new property with namespace in XNJS'
idbElement = etree.Element('{http://www.fz-juelich.de/unicore/xnjs/engine}Property')
idbElement.set('name', 'XNJS.idbfile')
idbElement.set('value', '/etc/unicore/idb')
propsElements = xnjsDoc.xpath('/eng:EngineConfiguration/eng:Properties', namespaces = xnjsNSPrefixMap)
assert len(propsElements) == 1
propsElements[0].append(idbElement)
xnjsXmlResultExpected = '<eng:EngineConfiguration xmlns:eng="http://www.fz-juelich.de/unicore/xnjs/engine">\n\
  <eng:Core>\n\
    <eng:ProcessorFactory>de.fzj.unicore.xnjs.ems.processors.DefaultProcessorFactory</eng:ProcessorFactory>\n\
    <eng:FileTransferEngine>de.fzj.unicore.xnjs.io.simple.SimpleFileTransferEngine</eng:FileTransferEngine>\n\
  </eng:Core>\n\
  <eng:ProcessingChain actionType="JSDL" jobDescriptionType="{http://schemas.ggf.org/jsdl/2005/11/jsdl}JobDefinition">\n\
    <eng:Processor>de.fzj.unicore.xnjs.jsdl.JSDLProcessor</eng:Processor>\n\
    <eng:Processor>de.fzj.unicore.xnjs.ems.processors.UsageLogger</eng:Processor>\n\
    <eng:Processor>pl.edu.icm.unicore.accounting.processor.AccountingJobProcessor</eng:Processor>\n\
  </eng:ProcessingChain>\n\
  <eng:Properties>\n\
    <eng:Property name="RUS.bssMachine" value="goofy.grid"/>\n\
    <eng:Property name="XNJS.idbfile" value="/etc/unicore/idb"/>\n\
  </eng:Properties>\n\
</eng:EngineConfiguration>\n'
xnjsResult = etree.tostring(xnjsDoc,
	encoding = 'utf-8', method = 'xml', pretty_print = True, xml_declaration = False)
assert xnjsResult == xnjsXmlResultExpected
print ' - result: OK'


'''
	DONE
'''
print 'Done'

